!!ARBfp1.0
# computes 2D noise into red channel 
# + z and w gradient terms into green and blue channels
#
# use with texgen.vs vertex shader to generate noise texture

ATTRIB noiseInput = fragment.texcoord;

# base for blum blum shub should be product of two primes
# but I couldn't find any product that looked good and didn't
# result in huge texture sizes. 61 is a single prime that seemed 
# to produce reasonably good results
PARAM base = {61., 0.0163934};		# base, 1/base


# integer and fractional components of position
TEMP intArg, fracArg;
FLR intArg.xy,  noiseInput;
FRC fracArg.xy, noiseInput;

# compute fade values from fractional location in grid cell
# fade(t) = t*t*t*(t*(6*t-15)+10)
TEMP fade;
MAD fade.xy,fracArg,-2,3;		# old fade function t*t*(-2*t+3)
MUL fade.xy,fade,fracArg;
MUL fade.xy,fade,fracArg;

# hash will hold hash values for x,y; x+1,y; x,y+1 and x+1,y+1
TEMP hash, htmp;

# start with y & y+1 terms and run through hash function

# I scaled by a number relatively prime to base to bootstrap it into
# the more interesting part of the hash function a better choice may
# have been to just apply the hash multiple times. Note that
# this is JUST for hash purposes, it does not scale the frequency of
# the noise
MAD hash.xy,intArg.y,7,{0,7};	# 7*{y, y+1}
MUL htmp.xy,hash,base.y;	#   mod base
FLR htmp.xy,htmp;
MAD hash.xy,htmp,-base.x,hash;
MUL hash.xy,hash,hash;		#   squared
MUL htmp.xy,hash,base.y;	#   mod base
FLR htmp.xy,htmp;
MAD hash.xy,htmp,-base.x,hash;

# add x & x+1 terms and run through hash again
ADD hash,intArg.x,hash.xxyy;	# + {x, x+1}
ADD hash,hash,{0,1,0,1};
MUL hash,hash,hash;		#   squared
MUL htmp,hash,base.y;		#   mod base
FLR htmp,htmp;
MAD hash,htmp,-base.x,hash;

# extract +/- gradient bits each axis
TEMP xs, ys, zs, ws;
MUL xs,hash,.5;
FRC xs,xs;
SGE xs,xs,.49999;

MUL ys,hash,.25;
FRC ys,ys;
SGE ys,ys,.49999;

MUL zs,hash,.125;
FRC zs,zs;
SGE zs,zs,.49999;

MUL ws,hash, 0.0625;
FRC ws,ws;
SGE ws,ws,.49999;

# construct full x & y gradients
TEMP g, gx, gy;
ADD g, fracArg.xxyy, {0,-1,0,-1};
LRP gx, xs, -g.xyxy, g.xyxy;
LRP gy, ys, -g.zzww, g.zzww;
ADD g, gx, gy;

# blend x/y noise and z/w gradient terms in x
TEMP noise;
LRP noise.xy, fade.x, g.ywyw, g.xzxz;
LRP g.xy,     fade.x,zs.ywyw,zs.xzxz;
LRP g.zw,     fade.x,ws.ywyw,ws.xzxz;


# blend noise and z/w gradient terms in y
LRP noise.x,  fade.y, noise.y, noise.x;
LRP noise.yz, fade.y, g.ywyw,  g.xzxz;


# scale into final color (several choices, uncomment only one)
MAD result.color, noise, {.5,1,1,0}, {.5,0,0,1}; # all three at once
#MAD result.color, noise.x, .5, .5;		 # just the noise
#MOV result.color, noise.y;			 # just the z gradient
#MOV result.color, noise.z;			 # just the w gradient


END